/*
 * This File is part of the iowj-project   
 * $Id: IowBatch.java,v 1.1.1.1 2006/01/15 16:47:26 Thomas Wagner Exp $
 * Copyright (C)2005 by Thomas Wagner
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.wagner_ibw.appl;

import java.util.Date;
import java.util.NoSuchElementException;

import de.wagner_ibw.iow.IowFactory;
import de.wagner_ibw.iow.Iow24;
import de.wagner_ibw.iow.Iow40;
import de.wagner_ibw.iow.IowPort;
import de.wagner_ibw.iow.IowPortChangeListener;

/**
 * This simple application starts a executable if the specified
 * bit on port chages its state.
 * @author Thomas Wagner
 */
public class IowBatch implements IowPortChangeListener {

	private static String  batch;
	private static int     chip;
	private static int     port;
	private static int     bit;
	private static IowFactory     devs;
	private static Iow24   dev24;
	private static Iow40   dev40;
	private static boolean state;
	
	public IowBatch() {
		init();
	}
		
	private void init() {
		devs  = IowFactory.getInstance();
		if (devs.getNumDevices() == 0) {
			System.out.println("Cannot find any connected IOW device(s)!");
			devs.exit(-1);
		}
		System.out.println("Try to activate batch on IOW" + chip + " P" + port+"." + bit);
		if (chip == 24) {
			try {
				dev24 = devs.getIow24Device();
			} catch (NoSuchElementException e) {
				System.out.println("No IOW24 device found!");
				devs.exit(-1);
			}
			
			dev24.scanPorts();
			if(dev24.getPort(port).isBitClear(bit)) {
				state = false;
			}
			else {
				state = true;
			}
			dev24.autonomous(true);	
			dev24.getPort(port).addPortChangeListener(this);
		}
		else {
			try {
				dev40 = devs.getIow40Device();
			} catch (NoSuchElementException e) {
				System.out.println("No IOW40 device found!");
				devs.exit(-1);
			}
			
			dev40.scanPorts();
			if(dev40.getPort(port).isBitClear(bit)) {
				state = false;
			}
			else {
				state = true;
			}
			dev40.autonomous(true);	
			dev40.getPort(port).addPortChangeListener(this);
		}
		System.out.println("Current Pin Status: " + state);
	} 
	
	private static void usage(String msg) {
		System.out.println(msg);
		System.out.println("Usage: java IowBatch <chip> <port> <bit> <cmd>");
		System.out.println("  <chip> type of IO-Warrior: 24|40");
		System.out.println("  <port> scanned port of the IO-Warrior: 0...1 or 0...3");
		System.out.println("  <bit>  scanned bit  of the IO-Warrior: 0...7");
		System.out.println("  <cmd>  executable that have to be execute if the pin status changes");
		System.out.println("e.g. 'java IowBatch 24 0 1 notepad'");
	}

	public static void main(String[] args) {
		
		if(args.length != 4) {
			usage("Missing or too many arguments!");
			System.exit(-1);
		}
		
		try {
			chip = Integer.parseInt(args[0]);
			if (!(chip == 24 || chip == 40)) {
				usage("Wrong argument <chip> '" + chip + "'!");
				System.exit(-1);
			}
		} catch (NumberFormatException e) {
			usage("Wrong argument <chip> '" + args[0] + "'!");
			System.exit(-1);
		}
		
		try {
			port = Integer.parseInt(args[1]);
			if (chip == 24) {
				if ( port > 1 || port < 0) {
					usage("Wrong argument <port> '" + port + "'!");
					System.exit(-1);
				}
			} else {
				if ( port > 3 || port < 0) {
				 	usage("Wrong argument <port> '" + port + "'!");
					System.exit(-1);
				}
			}
		} catch (NumberFormatException e) {
			usage("Wrong argument <port> '" + args[1] + "'!");
			System.exit(-1);
		}
		
		try {
			bit = Integer.parseInt(args[2]);
			if ( bit > 7 || port < 0) {
				usage("Wrong argument <bit> '" + bit + "'!");
				System.exit(-1);
			}
		} catch (NumberFormatException e) {
			usage("Wrong argument <port> '" + args[2] + "'!");
			System.exit(-1);
		}	
	
		batch = args[3];
				
		IowBatch batch = new IowBatch();
	}
	
	/* (non-Javadoc)
	 * @see de.wagner_ibw.iow.IowPortChangeListener#portChanged(de.wagner_ibw.iow.IowPort)
	 */
	public void portChanged(IowPort cPort) {
		//*deb*/System.out.println("Port changes ...");
		if (cPort.getIndex() == port) {
			if (cPort.isBitSet(bit) != state) {
				Date date = new Date(System.currentTimeMillis());
				System.out.println("Start batch '"+ batch +"' at " + date + " ...");
				try {
					Runtime.getRuntime().exec(batch);
				} catch (Exception e) {
					System.out.println("Start batch '"+ batch +" failed! " + e);
					devs.exit(-1);
				}
			}
		}
	}
}
