/* 
 * This File is part of the iowj-project   
 * $Id: SwitchMatrix.java,v 1.1.1.1 2006/01/15 16:47:27 Thomas Wagner Exp $
 * Copyright (C)2005 by Thomas Wagner
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.wagner_ibw.iow.smx;

import java.util.Hashtable;
import java.util.Vector;

import sun.misc.Queue;
import de.wagner_ibw.iow.AbstractIowDevcie;
import de.wagner_ibw.iow.SpecialModeFunction;

/**
 * This is the implementation of the IO Warrior 40 key matrix special mode function.
 * <p><b>(unfinished!)</b></p>
 * @author Thomas Wagner
 * @since 0.9.4
 */
public class SwitchMatrix implements SpecialModeFunction {

	/**
	 * Name of this special mode function.
	 */
	final static String name = "SMX";
	
	private Vector listeners;

	/**
	 * Reference to the underlying IOW device.
	 */
	private AbstractIowDevcie iow;

	/**
	 * Current state of the switch matrix.
	 */	
	private int matrix[]    = {0,0,0,0,0,0,0,0};
	private int oldMatrix[] = {0,0,0,0,0,0,0,0};

	private Queue currentMatrixStatus;
	private boolean doScan = false;

	private Hashtable keyMap = null;

	/**
	 * Constructor
	 */
	public SwitchMatrix(){
		listeners = new Vector();
		currentMatrixStatus = new Queue();
	}
	
	public void addSwitchMatrixChangeListener(SwitchMatrixChangeListener smcl) {
		if(!listeners.contains(smcl))
			listeners.add(smcl);
	}

	public void removeSwitchMatrixChangeListener(SwitchMatrixChangeListener smcl) {
		if(listeners.contains(smcl))
			listeners.remove(smcl);
	}
	
	class NotifierThread extends Thread {

		SwitchMatrixEvent event = null;
		SwitchMatrixChangeListener listener = null;

		NotifierThread(SwitchMatrixEvent event, SwitchMatrixChangeListener listener) {
			this.event = event;
			this.listener = listener;
		}
	
		public void run() {
			listener.matrixChanged(event);
		}
	}
	
	/**
	 * Returns Information about the switch matrix as String.
	 * @return String representation of information about the switch matrix
	 * (row7,row6, ... row0).
	 */
	public String toString(){
		StringBuffer sb = new StringBuffer();
		sb.append("SMX[");
		for (int i = 7; i > -1; i--) {
			sb.append(matrix[i]);
			if(i > 0)
				sb.append(",");
		}
		sb.append("],");
		long[] longValue = getLongValue();
			sb.append(Long.toHexString(longValue[1]));
		sb.append(",");
		sb.append(Long.toHexString(longValue[0]));
		sb.append(",");
		sb.append(",oSMX[");
		for (int i = 7; i > -1; i--) {
			sb.append(oldMatrix[i]);
			if(i > 0)
				sb.append(",");
		}
		sb.append("]");	
		return sb.toString();
	}
	
	/* (non-Javadoc)
	 * @see de.wagner_ibw.iow.SpecialModeFunction#getDisableReport()
	 */
	public int[] getDisableReport() {
		int wbuf[] = {0x18,0,0,0,0,0,0,0};
		return wbuf;
	}

	/* (non-Javadoc)
	 * @see de.wagner_ibw.iow.SpecialModeFunction#getEnableReport()
	 */
	public int[] getEnableReport() {
		int wbuf[] = {0x18,1,0,0,0,0,0,0};
		return wbuf;
	}

	/* (non-Javadoc)
	 * @see de.wagner_ibw.iow.SpecialModeFunction#getIowSpecialBits()
	 */
	public int[] getIowSpecialBits(int deviceIdentifier) {
		if(deviceIdentifier == AbstractIowDevcie.IOW24ID){
			int masks[] = {}; //no special mode availiable
			return masks;
		} else {
			int masks[] = {0x00, 0x00, 0xff, 0xff};
			return masks;
		}
	}

	/* (non-Javadoc)
	 * @see de.wagner_ibw.iow.SpecialModeFunction#getReportIds()
	 */
	public int[] getReportIds() {
		int ids[] = {0x19,0x1a};
		return ids;
	}

	/* (non-Javadoc)
	 * @see de.wagner_ibw.iow.SpecialModeFunction#getSpecialModeFuncionId()
	 */
	public int getSpecialModeFuncionId() {
		return SpecialModeFunction.SMF_SMX_ID;
	}

	/* (non-Javadoc)
	 * @see de.wagner_ibw.iow.SpecialModeFunction#matchReportId(int)
	 */
	public boolean matchReportId(int reportId) {
		return ((reportId == 0x19) || (reportId == 0x1a));
	}

	/* (non-Javadoc)
	 * @see de.wagner_ibw.iow.SpecialModeFunction#reportReceived(int[])
	 */
	public void reportReceived(int[] readBuffer) {
		if (doScan) {
			currentMatrixStatus.enqueue(readBuffer);
		}
		else {
			
			setMatrix(readBuffer);
		}
	}

	/* (non-Javadoc)
	 * @see de.wagner_ibw.iow.SpecialModeFunction#setIowDevice(de.wagner_ibw.iow.IowInterface)
	 */
	public void setIowDevice(AbstractIowDevcie iow) {
		this.iow = iow;
	}

	/* (non-Javadoc)
	 * @see de.wagner_ibw.iow.SpecialModeFunction#checkCompatibility()
	 * @since 0.9.4
	 */
	public String checkCompatibility(int id, int rev, int specialModes) {
		int atLeastRev = 0x1021;
		//only for IO-Warrior 40
		if(id != AbstractIowDevcie.IOW40ID) return "Works only with IOW40";
		//only for Revision 1.0.2.1 and newer
		if(rev < atLeastRev) return "Revision is lesser than 1.0.2.1";
		return null;
	}

	/* (non-Javadoc)
	 * @see de.wagner_ibw.iow.SpecialModeFunction#getName()
	 */
	public String getName() {
		return name;
	}

	public int[] scanMatrix() {
		doScan = true;
		int wbuf[] = {0x19,0,0,0,0,0,0,0};
		long ret = iow.writeReport(1,wbuf);
		if(ret == 8){
			try {
				setMatrix((int[])currentMatrixStatus.dequeue());
				setMatrix((int[])currentMatrixStatus.dequeue());
				doScan = false;
			}
			catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		return matrix;
	}
	
	public long[] getLongValue() {
		long ret[] = {0,0};
		for (int i = 0; i < 4; i++){
			long newval = (long) matrix[i] << (i * 8);
			ret[0] = ret[0] + newval;
		}
		ret[0] = ret[0] & 0xffffffff; 
		for (int i = 3; i < matrix.length; i++){
			long newval = (long) matrix[i] << (i * 8);
			ret[1] = ret[1] + newval;
		}
		ret[1] = ret[1] & 0xffffffff;
		return ret;
	}

	private void setMatrix(int[] readBuffer) {
		//first report
		if(readBuffer[0] == 0x1a) {
			//*deb*/System.out.println("setting matrix[4...7]");
			matrix[4] = readBuffer[1];
			matrix[5] = readBuffer[2];
			matrix[6] = readBuffer[3];
			matrix[7] = readBuffer[4];
		}
		//second report
		else if(readBuffer[0] == 0x19) {
			//*deb*/System.out.println("setting matrix[0...3]");
			matrix[0] = readBuffer[1];
			matrix[1] = readBuffer[2];
			matrix[2] = readBuffer[3];
			matrix[3] = readBuffer[4];
			
			//matrix eingelesen
			if(!doScan) {
				/*deb*/System.out.println(this);
				SwitchMatrixEvent event = new SwitchMatrixEvent(matrix, oldMatrix);
				for(int i = 0; i < listeners.size(); i++) {
					SwitchMatrixChangeListener smcl = (SwitchMatrixChangeListener)listeners.get(i);
					(new NotifierThread(event, smcl)).start();
				}
			}
				
			for (int i = 0; i < 8; i++) {
				oldMatrix[i] = matrix[i];
			}
		}
		else {
			System.out.println("unexpected reportid: " + readBuffer[0]);
		}
	}
}
