/*
 * This File is part of the iowj-project   
 * $Id: SwitchMatrixKeyMap.java,v 1.1.1.1 2006/01/15 16:47:27 Thomas Wagner Exp $
 * Copyright (C)2004 by Thomas Wagner
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
package de.wagner_ibw.iow.smx;

/**
 * It is a simple example program for the switch matrix.
 * 
 * @author Thomas Wagner
 * @since 0.9.4
 *
 */
public class SwitchMatrixKeyMap {
		
	private String[][] label = new String[8][8];
	private int xs = 8;
	private int ys = 8;
		
	public  SwitchMatrixKeyMap(String map[][]) {
		loadMatrix(map);
	}

	public void setKeyMap(String map[][]) {
		loadMatrix(map);
	}
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("KeyMatrix[x=");
		sb.append(xs);
		sb.append("][y=");
		sb.append(ys);
		sb.append("]:\n");
		for (int x = 0; x < label.length; x++ ) {
			for (int y = 0; y < label[x].length; y++) {
				sb.append(label[x][y] + " ");
			}
			sb.append("\n");
		}
		return sb.toString();
	}
	
	public String getLabel(int[] matrix) {
		int x = -1;
		int y = -1;
		for( y = 0; y < ys; y++){
			if (matrix[y] != 0){
				if(matrix[y] == 0x01) x = 0;
				else if(matrix[y] == 0x02) x = 1;
				else if(matrix[y] == 0x04) x = 2;
				else if(matrix[y] == 0x08) x = 3;
				else if(matrix[y] == 0x10) x = 4;
				else if(matrix[y] == 0x20) x = 5;
				else if(matrix[y] == 0x40) x = 6;
				else x = 7;
				break;	//found
			}
		}
		if (x != -1 && y != -1) {
			String retLabel = label[x][y];
			if (retLabel == null) {
				retLabel = "no label defined";
			}
			/*deb*/System.out.println("found ("+x+","+y+"): "+ retLabel);
			return retLabel;
		}
		else {
			return "no key pressed";
		}
	}
	
	public String getLabel(SwitchMatrixEvent event) {
		return getLabel(event.getNewMatrix());
	}
		
	public static void main(String[] args) {
		String test[][] = {{"0","1","2","3"},{"4","5","6","7"},{"8","9","A","B"},{"C","D","E","F"}};
	
		SwitchMatrixKeyMap smkm = new SwitchMatrixKeyMap(test);
		System.out.println(smkm);
		System.out.println("1 : " +smkm.getLabel(new int[] {1,0,0,0,0,0,0,1}));
		System.out.println("2 : " +smkm.getLabel(new int[] {2,0,0,0,0,0,0,2}));
		System.out.println("3 : " +smkm.getLabel(new int[] {4,0,0,0,0,0,0,4}));
		System.out.println("4 : " +smkm.getLabel(new int[] {8,0,0,0,0,0,0,8}));
		System.out.println("5 : " +smkm.getLabel(new int[] {0,1,0,0,0,0,1,0}));
		System.out.println("6 : " +smkm.getLabel(new int[] {0,2,0,0,0,0,2,0}));
		System.out.println("7 : " +smkm.getLabel(new int[] {0,4,0,0,0,0,4,0}));
		System.out.println("8 : " +smkm.getLabel(new int[] {0,8,0,0,0,0,8,0}));
		System.out.println("9 : " +smkm.getLabel(new int[] {0,0,1,0,0,0,0,0}));
		System.out.println("10: " +smkm.getLabel(new int[] {0,0,2,0,0,0,0,0}));
		System.out.println("11: " +smkm.getLabel(new int[] {0,0,4,0,0,0,0,0}));
		System.out.println("12: " +smkm.getLabel(new int[] {0,0,8,0,0,0,0,0}));
		System.out.println("13: " +smkm.getLabel(new int[] {0,0,0,1,0,0,0,0}));
		System.out.println("14: " +smkm.getLabel(new int[] {0,0,0,2,0,0,0,0}));
		System.out.println("15: " +smkm.getLabel(new int[] {0,0,0,4,0,0,0,0}));
		System.out.println("16: " +smkm.getLabel(new int[] {0,0,0,8,0,0,0,0}));
		
		String[][] myMap = {{"1","2","3"},
							{"4","5","6"}};
		int[] matrix = {0,0,0,0,0,0,0,0};
		
		System.out.println(smkm);
		
				matrix[0] = 0x01;
				matrix[1] = 0;
				matrix[2] = 0;
				System.out.println("1: "+smkm.getLabel( matrix));
		
				matrix[0] = 0;
				matrix[1] = 0x01;
				matrix[2] = 0;
				System.out.println("2: "+smkm.getLabel( matrix));
		
				matrix[0] = 0;
				matrix[1] = 0;
				matrix[2] = 0x01;
				System.out.println("3: "+smkm.getLabel( matrix));
		
				matrix[0] = 0x02;
				matrix[1] = 0;
				matrix[2] = 0;
				System.out.println("4: "+smkm.getLabel( matrix));
		
				matrix[0] = 0;
				matrix[1] = 0x02;
				matrix[2] = 0;
				System.out.println("5: "+smkm.getLabel( matrix));
		
				matrix[0] = 0;
				matrix[1] = 0;
				matrix[2] = 0x02;
				System.out.println("6: "+smkm.getLabel( matrix));
		
				matrix[0] = 0;
				matrix[1] = 0;
				matrix[2] = 0;
				System.out.println("t: "+smkm.getLabel( matrix));
	}

	private void loadMatrix(String map[][]){
		xs= map.length;
		if ( xs> 7 )
			throw new IllegalArgumentException("Array to big (x more than 8 elements!");
		ys= map[0].length;
		if ( ys> 7 )
			throw new IllegalArgumentException("Array to big (y more than 8 elements!");
		
		for (int x = 0; x < map.length; x++ ) {
			for (int y = 0; y < map[x].length; y++) {
				label[x][y] = map[x][y];
			}
		}
	}
	
	
}
